#include <htc.h>
#include "wait_time.h"
#include "user_define.h"

#define	LCD_DB4				RB7
#define	LCD_DB5				RB6
#define	LCD_DB6				RB3
#define	LCD_DB7				RB2
#define	LCD_RS				RB4
#define	LCD_E				RB5	

#define	LCD_CMD_WAIT		200  /*400*/
#define	LCD_BIT_WAIT		5   /*10*/

extern  unsigned char   ms_mode;
extern  unsigned char	step_flg;
extern  unsigned char	out_flg;

/*---------------LCD MODULE----------------*/
void	lcd_db(unsigned char data)
{
	LCD_DB4=data&0x01;
	NOP();
	LCD_DB5=(data&0x02)>>1;
	NOP();
	LCD_DB6=(data&0x04)>>2;
	NOP();
	LCD_DB7=(data&0x08)>>3;
	wait_time(1);
}
void	lcd_wait(void)
{
	wait_time(2500);
}
void	lcd_c_set(void)
{
	LCD_RS=0;
	NOP();
	LCD_E=1;
	wait_time(LCD_BIT_WAIT);
	LCD_E=0;
	NOP();
}
void	lcd_d_set(void)
{
	LCD_RS=1;
	NOP();
	LCD_E=1;
	wait_time(LCD_BIT_WAIT);
	LCD_E=0;
	NOP();
}
void	lcd_cmd(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_c_set();
	lcd_db(data&0x0f);
	lcd_c_set();
	wait_time(LCD_CMD_WAIT);
}
void	lcd_data(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_d_set();
	lcd_db(data&0x0f);
	lcd_d_set();
	wait_time(LCD_CMD_WAIT);
}
void	lcd_init(void)
{
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(2);
	lcd_c_set();
	lcd_wait();
	
	lcd_cmd(0x28);
	lcd_cmd(0x08);
	lcd_cmd(0x0c);
	lcd_cmd(0x06);
}
void	lcd_clear(void)
{
	lcd_cmd(0x01);
}
void	lcd_xy(unsigned char x,unsigned char y)
{
unsigned char data;

	data=0x80;
	switch (y){
		case 0:data=0x80;break;
		case 1:data=0xc0;break;
		case 2:data=0x90;break;
		case 3:data=0xd0;break;
	}
	data=data+x;
	lcd_cmd(data);
}
void	lcd_print(const char *data)
{
	while (*data!=0){
		lcd_data(*data++);
	}
}
void	lcd_su(unsigned char data)
{
	switch (data){
		case 0:lcd_print("0");break;
		case 1:lcd_print("1");break;
		case 2:lcd_print("2");break;
		case 3:lcd_print("3");break;
		case 4:lcd_print("4");break;
		case 5:lcd_print("5");break;
		case 6:lcd_print("6");break;
		case 7:lcd_print("7");break;
		case 8:lcd_print("8");break;
		case 9:lcd_print("9");break;
		case 10:lcd_print("A");break;
		case 11:lcd_print("B");break;
		case 12:lcd_print("C");break;
		case 13:lcd_print("D");break;
		case 14:lcd_print("E");break;
		case 15:lcd_print("F");break;
	}
}
void	lcd_pr_byte(unsigned char data)
{
unsigned char data2;

	data2=data%16;data=data/16;
	lcd_su(data);
	lcd_su(data2);
}

void	lcd_1keta10(unsigned char count)
{
unsigned char n0;

	n0=(unsigned char)(count%10);

	lcd_su(n0);
}

void	lcd_2keta10(unsigned char count)
{
unsigned char n0,n1;

	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);

	lcd_su(n1);
	lcd_su(n0);
}

void	lcd_hz(unsigned long hz)
{
unsigned char n0,n1,n2,n3,n4,n5,n6;
unsigned char flg;
unsigned long count;

    flg=0;
	count=hz;
	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);count=(count-n3)/10;
	n4=(unsigned char)(count%10);count=(count-n4)/10;
 	n5=(unsigned char)(count%10);count=(count-n5)/10;
	n6=(unsigned char)(count%10);    
	
    lcd_xy(7,1);
	if((flg==0)&&(n6==0)){lcd_print(" ");}else{lcd_su(n6);flg=1;}
	if((flg==0)&&(n5==0)){lcd_print(" ");}else{lcd_su(n5);flg=1;}
	if((flg==0)&&(n4==0)){lcd_print(" ");}else{lcd_su(n4);flg=1;}
	if((flg==0)&&(n3==0)){lcd_print(" ");}else{lcd_su(n3);flg=1;}
	if((flg==0)&&(n2==0)){lcd_print(" ");}else{lcd_su(n2);flg=1;}
 	if((flg==0)&&(n1==0)){lcd_print(" ");}else{lcd_su(n1);flg=1;}
	lcd_su(n0);	
}

void    lcd_range(unsigned char no)
{
    lcd_xy(0,1);
    lcd_print("RANGE");
    lcd_1keta10(no+1);
}

void    lcd_mode(void)
{
    lcd_xy(0,0);
    if(ms_mode==0){
        if(step_flg==0){
            lcd_print("CONTROL ");
            if(out_flg==1){
                lcd_xy(7,0);
                lcd_print("*");
            }
        }
        else{
            lcd_print("STEP    ");
            if(out_flg==1){
                lcd_xy(7,0);
                lcd_print("*");
            }
        }
    }
    if(ms_mode==1){
        lcd_print("SWEEP   ");
    }
    
}

void    lcd_level(unsigned long data)
{
unsigned long   out_data;
unsigned char   nn;
unsigned char   n0,n1,n2,n3;

    //out_data=data*19;
    //out_data=out_data*50;
    //out_data=out_data*5;
    //out_data=out_data/256;
    //if(out_data>6800){
    //    nn=0;
    //    out_data=out_data-6800;
    //}
    //else{
    //    nn=1;
    //    out_data=6800-out_data;
    //}

    out_data=data*25;
    out_data=out_data*27;
    out_data=out_data/32;
    if(out_data>7500){
        nn=0;
        out_data=out_data-7500;
    }
    else{
        nn=1;
        out_data=7500-out_data;
    }


	n0=(unsigned char)(out_data%10);out_data=(out_data-n0)/10;
	n1=(unsigned char)(out_data%10);out_data=(out_data-n1)/10;
    n2=(unsigned char)(out_data%10);out_data=(out_data-n2)/10;
    n3=(unsigned char)(out_data%10);
    
    lcd_xy(10,0);
    lcd_su(n3);
    lcd_su(n2);
    lcd_xy(13,0);
    lcd_su(n1);
    lcd_xy(9,0);
    if(nn==1){lcd_print("-");}else{lcd_print("+");}
}